import pygame
from pygame.locals import *
import time

class BusySprite(pygame.sprite.Sprite):
    """
    Busy sprite:
    Sprite that makes the stage "busy".
    May or may not display an image on the screen.
    """
    pass

class TimeoutSprite(BusySprite):
    """
    Timeout sprite:
    Sprite that makes the stage "busy" and times out after a given amount of seconds.
    May or may not display an image on the screen.
    """

    def __init__(self, timeout, on_start = None, on_kill = None):
        BusySprite.__init__(self)

        self.timeout = timeout
        self.on_start = on_start
        self.on_kill = on_kill
        self.liveuntil = None

    def update(self, *args):
        # first update is when we initialize "liveuntil" and start the countdown
        if self.liveuntil is None:
            if self.on_start is not None:
                self.on_start()
            self.liveuntil = time.time() + self.timeout

        if (time.time() >= self.liveuntil) or (args == ("User bored",)):
            if self.on_kill is not None:
                self.on_kill()
            self.kill()