from talktext import *
from game_object import *
import random

class Character(GameObject):
    TALK_ANIM_SPEED = (90, 250) # milliseconds (RANDRANGE TUPLE) to change image - the bigger this is, the slower the player is animated
    
    def __init__(self, name, talkcolor, walkpos = None, walkdir = None):
        GameObject.__init__(self, name, walkpos, walkdir)

        self.talkcolor = talkcolor

        self.finish_talk() # reset talk parameters, just in case

    def talk(self, text):
        return TalkText(text, self.talkcolor, self.rect, self.start_talk, self.finish_talk)

    def update_talk_anim(self):
        if pygame.time.get_ticks() >= self.talk_tick:
            # swap frame
            self.talk_anim_in_talk_frame = not self.talk_anim_in_talk_frame
            # determine when the next time is going to be
            self.talk_tick = pygame.time.get_ticks() + random.randrange(*self.TALK_ANIM_SPEED)

            # actually swap frame
            if self.talk_anim_in_talk_frame:
                self.image = self.get_talk_image()
            else:
                self.image = self.get_stand_image()

    def stand_still(self):
        self.image = self.get_stand_image()

    def start_talk(self):
        self.talking = True
        self.talk_tick = pygame.time.get_ticks() # to ensure change on first update_talk_anim
        self.talk_anim_in_talk_frame = False # to ensure change to True on first update_talk_anim
        self.update_talk_anim() # first update_talk_anim

    def finish_talk(self):
        self.talking = False
        self.talk_tick = 0
        self.talk_anim_in_talk_frame = False
        self.stand_still() # de-animate

    def update(self, *args):
        if self.talking:
            self.update_talk_anim()