from cursor import *
from game_stages import *
from textline import *
from talkpane import *
from inventorypane import *
from object_in_hand import *
import game_vars

class GameScreen(object):
    TRANSITIONS = {
        # from street left
        (176, 96, 162)  : (TouristInformationCenter, ((94, 359), DIR_RIGHTBACK)),
        #(58, 170, 53)   : (Park, ((136, 352), DIR_RIGHTBACK)),
        (58, 170, 53)   : (CandyStore, ((100, 331), DIR_RIGHTBACK)),
        (27, 93, 27)    : (StreetRight, ((20, 335), DIR_RIGHT)),
        (185, 122, 87)  : (Dump, ((597, 341), DIR_LEFT)),

        # from street right
        (242, 141, 79)  : (StreetLeft, ((612, 335), DIR_LEFT)),
        (159, 7, 55)    : (LeShirt, ((43, 343), DIR_RIGHTBACK)),

        # from tourist information center
        (252, 255, 0)   : (StreetLeft, ((88, 313), DIR_FRONT)),

        # from leshirt
        (0, 70, 100)    : (StreetRight, ((137, 320), DIR_FRONT)),

        # from dump
        (0, 154, 147)   : (StreetLeft, ((20, 335), DIR_RIGHT)),

        # from park
        (255, 0, 0)     : (StreetLeft, ((389, 313), DIR_FRONT)),

        # from candy store
        (42, 255, 0)    : (StreetLeft, ((389, 313), DIR_FRONT))
    }

    def __init__(self, screen):
        self.display_flags = 0

        self.screen = screen

        self.verb = VERB_WALK

        self.cursors = { VERB_WALK : Cursor("walk_cursor.png", (1, 1)),
                         VERB_USE : Cursor("use_cursor.png", (3, 3)),
                         VERB_LOOK : Cursor("look_cursor.png", (1, 1)),
                         VERB_TALK : Cursor("talk_cursor.png", (0, 3)),
                         VERB_CROSSHAIR : Cursor("use_cursor.png", (3, 3)) }

        self.cursor = pygame.sprite.RenderUpdates(self.cursors[self.verb])

        self.hitobject = None
        self.object_in_hand = None

        self.stage = game_vars.Vars.stage(game_vars.Vars.player_pos, game_vars.Vars.player_dir)

        self.background_music_filename = None
        self.background_music_channel = None
        self.update_background_music()
        
        self.textline = pygame.sprite.RenderUpdates(Textline())
        self.textline.update(self.verb, self.hitobject, self.object_in_hand)

        self.inventorypane = InventoryPane()
        self.talkpane = TalkPane()

    def refresh_entire_screen(self):
        self.stage.draw_background(self.screen)
        self.stage.draw_foreground(self.screen)

        if self.stage.is_busy():
            self.talkpane.draw(self.screen)
        else:
            self.textline.draw(self.screen)
            self.inventorypane.draw(self.screen)
        
        pygame.display.update()

    def stage_transition(self, stage_class, stage_args):
        game_vars.Vars.stage = stage_class
        self.stage = stage_class(*stage_args)
        self.update_background_music()

        # "first time" at this stage so refresh
        self.refresh_entire_screen()

    def update_background_music(self):
        if self.background_music_filename == self.stage.background_music_filename:
            return # Do not change the music

        self.background_music_filename = self.stage.background_music_filename

        if self.background_music_channel is not None:
            # stop the current music
            self.background_music_channel.stop()
            self.background_music_channel = None

        if self.background_music_filename is not None:
            # start the new music
            self.background_music_channel = pygame.mixer.Sound(os.path.join("data", self.background_music_filename)).play(loops=-1)

    def play(self):
        quit = False
        stage_busy = False

        # First time, refresh entire screen
        self.refresh_entire_screen()
        clock = pygame.time.Clock()

        while not quit:
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYUP and event.key == K_ESCAPE):
                    quit = True

                elif event.type == KEYUP and event.key == K_f:
                    self.display_flags ^= pygame.FULLSCREEN
                    self.screen = pygame.display.set_mode(SCREEN_SIZE, self.display_flags, 32)
                    self.refresh_entire_screen()

#                elif event.type == KEYUP and event.key == K_v:
#                    for item in dir(game_vars.Vars):
#                        if not item.startswith("_"):
#                            print "%s = %r" % (item, eval("game_vars.Vars.%s" % item))
#                    print

                elif event.type == MOUSEBUTTONDOWN and event.button == LEFT_BUTTON:
                    self.talkpane.update(event.type, event.pos)
                    if not self.stage.is_busy():
                        inv_item = self.inventorypane.click(event.pos, self.verb)
                        if inv_item is not None:
                            self.stage.use_item(self.verb, inv_item, self.object_in_hand)
                        if self.stage.click(event.pos, self.verb):
                            self.change_verb(VERB_WALK)

                elif event.type == MOUSEBUTTONDOWN and event.button == RIGHT_BUTTON:
                    if not self.stage.is_busy():
                        self.right_click()
                    else:
                        self.stage.user_bored()

                elif event.type == MOUSEMOTION:
                    self.update_hitobject(event.pos)
                    self.talkpane.update(event.type, event.pos)
                    self.inventorypane.update(event.type, event.pos)

            updating_stage = True
            while updating_stage:
                updating_stage = False # usually we only do this once...

                stage_update_return_value = self.stage.update(self.object_in_hand) # usually this is None

                if game_vars.Vars.won_game:
                    quit = True

                if isinstance(stage_update_return_value, TalkOptions): # we're talking
                    self.talkpane.set_talkoptions_object(stage_update_return_value)

                elif isinstance(stage_update_return_value, tuple): # we're transitioning, maybe
                    if self.TRANSITIONS.has_key(stage_update_return_value):
                        updating_stage = True
                        stage_busy = False
                        self.stage_transition(*self.TRANSITIONS[stage_update_return_value])

                elif isinstance(stage_update_return_value, type) and issubclass(stage_update_return_value, InventoryItem): # we got a new item
                    self.inventorypane.add_item(stage_update_return_value)

                elif isinstance(stage_update_return_value, ObjectInHand): # we're using an inventory item on something
                    self.object_in_hand = stage_update_return_value.object_in_hand

            self.talkpane.update()
            self.inventorypane.update()

            update_rects = []

            if stage_busy and not self.stage.is_busy():
                # Stage just became un-busy. Add the cursor, and make sure the talk pane rect is updated
                stage_busy = False
                self.cursor.empty()
                self.cursor.add(self.cursors[self.verb])
                update_rects.append(self.talkpane.rect)

                # Also it's important to update the textline again after the stage became un-busy, because
                # maybe the hitobject no longer exists in the stage! (Item was picked up!)
                self.update_hitobject(pygame.mouse.get_pos())

                # And change back to walk verb after becoming un-busy, unless we have an object in hand
                if self.object_in_hand is None:
                    self.change_verb(VERB_WALK)
                # ...in which case, at least refresh the text line
                else:
                    self.textline.update(self.verb, self.hitobject, self.object_in_hand)
            elif not stage_busy and self.stage.is_busy():
                # Stage just became busy. Remove the cursor, and make sure the talk pane rect is updated
                stage_busy = True
                self.cursor.empty()
                update_rects.append(self.talkpane.rect)

            if stage_busy:
                self.cursor.empty()
                if self.talkpane.is_in_options_mode():
                    self.cursor.add(self.cursors[VERB_CROSSHAIR])

            self.cursor.update(pygame.mouse.get_pos())

            update_rects += self.stage.draw(self.screen)
            if self.stage.is_busy():
                update_rects += self.talkpane.draw(self.screen)
            else:
                update_rects += self.textline.draw(self.screen)
                update_rects += self.inventorypane.draw(self.screen)
            update_rects += self.cursor.draw(self.screen)

            pygame.display.update(update_rects)

            clock.tick(FPS)

        if self.background_music_channel is not None:
            # stop the current music
            self.background_music_channel.stop()

    def change_verb(self, new_verb):
        self.cursor.remove(self.cursors[self.verb])
        self.verb = new_verb
        self.cursor.add(self.cursors[self.verb])
        self.object_in_hand = None
        self.textline.update(self.verb, self.hitobject, self.object_in_hand)
        
    def right_click(self):
        self.change_verb((self.verb + 1) % NUM_VERBS)

    def update_hitobject(self, pos):
        # Find hitobject on stage and in the inventory pane
        hitobject = self.stage.hitobject(pos)
        if hitobject is None:
            hitobject = self.inventorypane.hitobject(pos)

        if hitobject != self.hitobject:
            self.hitobject = hitobject
            self.textline.update(self.verb, self.hitobject, self.object_in_hand)
