from game_things import *
from delay import *
from object_in_hand import *
from talkoptions import *
import game_vars

class GameScript(object):
    @staticmethod
    def talk_default(player, object):
        yield player.talk("My mind hadn't gone THAT far.")

    @staticmethod
    def look_default(player, object):
        yield player.talk("My eyes aren't what they used to be.")

    @staticmethod
    def use_default(player, object):
        if isinstance(object, InventoryItem):
            yield ObjectInHand(object)
        else:
            yield player.talk("I'm not touching that.")

    @staticmethod
    def use_on_default(player, object, object_in_hand):
        if isinstance(object, NPC):
            yield player.talk("I don't want to give it away.")
        else:
            yield player.talk("Those things don't belong together.")

    @staticmethod
    def look_at_id_card(player, id_card):
        yield player.talk("It's completely ruined. You can't make anything out of it.")
        yield player.talk("I'm not even sure it's mine.")

    @staticmethod
    def look_at_myself(player, myself):
        yield player.talk("Wow, grandpa. You really let yourself go.")
        yield Delay(1)
        yield player.talk("Oh, right.")

    @staticmethod
    def talk_to_myself(player, myself):
        yield player.talk("I'm not talking to that guy.")
        yield Delay(1)
        player.set_stand_dir(DIR_FRONT)
        yield player.talk("He cheats in Yahtzee.")

    @staticmethod
    def use_myself(player, myself):
        yield player.talk("Gertrude?")

    @staticmethod
    def look_at_pothole(player, pothole):
        yield player.talk("Those things are dangerous. They cause car crashes.")

    @staticmethod
    def look_at_sign(player, sign):
        if isinstance(sign, TouristInformationCenterSign):
            yield player.talk("Municipal Tourist Information Center welcomes you to")
            yield player.talk("the dried zucchini capital of the world.")
        elif isinstance(sign, OptometristSign):
            yield player.talk("I can't quite make out what it says.")
            yield player.talk("Optimist? Opportunist? Osteoporosis?")
        elif isinstance(sign, LeShirtSign):
            yield player.talk("What's wrong with the clothes I got?")
            yield player.talk("A little oil and that zipper is good as new.")
        elif isinstance(sign, CandyStoreSign):
            yield player.talk("It's called \"Sweet & Sour\".")
            yield player.talk("I prefer sour. Sweets are bad for your teeth.")
            yield Delay(1)
            player.set_stand_dir(DIR_FRONT)
            yield player.talk("Now that I think about it, where ARE my teeth?!")

    @staticmethod
    def use_optometrist_door(player, door):
        yield player.talk("It doesn't seem to open.")

    @staticmethod
    def look_at_information_guy(player, guy):
        yield player.talk("Seems like a reliable person.")

    @staticmethod
    def give_candy_to_information_guy(player, guy, candy):
        if not game_vars.Vars.understand_the_information_guy_is_hungry:
            yield player.talk("I don't want to give it away.")
        else:
            yield guy.talk("Mmmmm... Ka-Boom! My favorite.")
            candy.kill()

            yield guy.talk("Thanks, I owe you one.")
            game_vars.Vars.he_owes_me_one = True

    @staticmethod
    def give_id_to_information_guy(player, guy, id):
        yield player.talk("Can you help me find out who I am?")
        
        if game_vars.Vars.he_owes_me_one:
            yield guy.talk("Well...")
            yield guy.talk("I have a friend who works at the DMV.")
            yield guy.talk("He can get you a new ID card.")
            yield player.talk("Okay, here you go.")
            yield Delay(1)
            yield guy.talk("Hold on.")
            yield guy.talk("I think I can just make out the address...")
            yield Delay(3)
            yield guy.talk("Wait a minute...")
            yield Delay(2)
            game_vars.Vars.won_game = True
        else:
            yield guy.talk("Beat it. I'm busy.")

    @staticmethod
    def talk_to_information_guy(player, guy):
        yield guy.talk("How may I help you, seal vows plate?")
        yield player.talk("I believe it's pronounced \"S'il vous plait\".")
        yield guy.talk("Whatever.")
        
        options = [
            "I need information.",
            "Do you have a map of the city?",
            "What book can help me find my way home?"
        ]

        if game_vars.Vars.think_the_information_guy_is_hungry:
            options.append("You seem to talk an awful lot about food.")

        options = TalkOptions(options)
        yield options

        yield player.talk(options.chosen_option_text)

        if options.chosen_option_index == 0:
            yield guy.talk("What would you like to know?")
            yield player.talk("I'm trying to remember who I am and where I live.")
            yield guy.talk("Well, I can't help you with that, but there is a guided tour\n" # hint - this is the biggest possible line
                           "leaving here every Tuesday morning.")
            yield guy.talk("You can see the great attractions of our city,")
            yield guy.talk("such as the city hall,")
            yield guy.talk("the mayor's private residence,")
            yield guy.talk("Founders Square,")
            yield guy.talk("and our world-renowned Zucchini Drying Institute\nand its luscious zucchini fields.")
            yield guy.talk("Also, free lunch for all participants of the tour.")
            yield guy.talk("Courtesy of the company's chef.")
            yield guy.talk("You're not allergic to feathers, are you?")
            yield player.talk("Does the tour pass anywhere near my house?")
            yield guy.talk("Are you the mayor?")
            yield player.talk("Not exactly...")
            yield guy.talk("Then no. The tour doesn't pass by your house.")
            game_vars.Vars.think_the_information_guy_is_hungry = True
        elif options.chosen_option_index == 1:
            yield guy.talk("Unfortunately, I just sold the last one.")
            yield player.talk("There are no maps at all?")
            yield guy.talk("Sorry, I thought you speak English.")
            yield guy.talk("No mapas leftos, capische?")
            yield guy.talk("All I have is my own copy,")
            yield guy.talk("and I need it to get here in the morning.")
        elif options.chosen_option_index == 2:
            yield guy.talk("Try the Guide to the Big Nags.")
            yield guy.talk("It's on the shelf, right next to the Map to the Old Farts.")
            yield player.talk("You have some attitude problems.")
            yield guy.talk("Sorry.")
            yield guy.talk("I'm used to talking to people who\ncan't understand the term \"old fart\".")
        elif options.chosen_option_index == 3:
            yield guy.talk("I missed my lunch, idiot.")
            yield Delay(2)
            yield guy.talk("Sorry. I get cranky when I'm hungry.")
            game_vars.Vars.understand_the_information_guy_is_hungry = True

    @staticmethod
    def look_at_bookshelf(player, bookshelf):
        if bookshelf.book_num == 0:
            yield player.talk("\"The City and You - a traveler's guide\"")
        elif bookshelf.book_num == 1:
            yield player.talk("\"Streets of Glory\"")
        elif bookshelf.book_num == 2:
            yield player.talk("\"The Amazing History of Dried Zucchini\"")
        elif bookshelf.book_num == 3:
            yield player.talk("\"Harry Potter and the Land of Zucchini\"")
            yield Delay(1.5)
            player.set_stand_dir(DIR_FRONT)
            yield player.talk("Who's Harry Potter?")

        bookshelf.book_num += 1
        bookshelf.book_num %= 4

    @staticmethod
    def look_at_tourists_in_tic(player, tourists):
        yield player.talk("He uses a German-to-English dictionary.")
        yield Delay(0.7)
        yield player.talk("I wonder if he's Belgian.")

    @staticmethod
    def look_at_balloon_in_tic(player, balloon):
        yield player.talk("An overblown bladder threatening to explode.")
        yield Delay(1)
        yield player.talk("I hope there's a bathroom in here.")

    @staticmethod
    def look_at_coat_hanger(player, hanger):
        yield player.talk("Thin, hard, and carries raggedy clothes.")
        yield Delay(1)
        player.set_stand_dir(DIR_FRONT)
        yield player.talk("*sigh*")
        yield player.talk("I miss Gertrude.")

    @staticmethod
    def look_at_coat(player, coat):
        yield player.talk("There's a piece of paper folded in its pocket.")

    @staticmethod
    def pick_up_coat(player, coat):
        yield coat.employee.talk("Please do not touch. No touching por favor.")

    @staticmethod
    def talk_to_tourists_in_tic(player, tourists):
        options = TalkOptions([
            "Excuse me?",
            "You're not from around here, are you?",
            "Do you happen to have a map of the city?",
            "My hovercraft is full of eels."
        ])

        yield options
        yield player.talk(options.chosen_option_text)

        if options.chosen_option_index == 0:
            yield tourists.talk("Ja?")
            yield player.talk("I wonder if you could help me.")
            yield player.talk("I'm lost and I can't remember my name or where I live.")
            yield player.talk("Maybe you could...")
            yield tourists.talk("Thirty days around plum tree. Then tortoise.")
            yield player.talk("Eh?")
            yield tourists.talk("Cone hat. Very cone hat.")
            yield Delay(1)
            player.set_stand_dir(DIR_FRONT)
            yield player.talk("Either I'm losing it, or he has\nthe worst conversation manual ever.")
        elif options.chosen_option_index == 1:
            yield tourists.talk("Sand angels in a bag of tuna.")
            yield player.talk("Oh, you're from Los Angeles.")
            yield tourists.talk("Throat.")
        elif options.chosen_option_index == 2:
            yield tourists.talk("Home fries with Bosnian aeroplane.")
            yield Delay(1)
            player.set_stand_dir(DIR_FRONT)
            yield player.talk("I think I need a German-to-English-to-English dictionary.")
        elif options.chosen_option_index == 3:
            yield tourists.talk("Bread cleansing?")

    @staticmethod
    def pick_up_clothes(player, clothes):
        if game_vars.Vars.got_needle:
            yield player.talk("Nah. I'm allergic to cloth.")
        else:
            yield player.talk("Ow! Something pricked me.")
            yield Delay(2)
            yield player.talk("It's a needle.")
            yield player.talk("I'd better take it away. Some kid might poke his eye out.")
            yield NeedleInv
            game_vars.Vars.got_needle = True

    @staticmethod
    def look_at_cash_register(player, cash_register):
        if game_vars.Vars.girl_in_store:
            yield cash_register.store.girl.talk("Hey! No messing around in there!")
        else:
            yield player.talk("I remember the cash register in dad's furniture store.")
            yield player.talk("Which is odd, considering he was a plumber.")
            if not game_vars.Vars.taken_twenty_dollar_bill:
                yield Delay(1.5)
                yield player.talk("There's something stuck underneath it.")
                game_vars.Vars.seen_twenty_dollar_bill = True

    @staticmethod
    def pick_up_cash_register(player, cash_register):
        if game_vars.Vars.girl_in_store:
            yield cash_register.store.girl.talk("Hey! No messing around in there!")
        else:
            if (not game_vars.Vars.seen_twenty_dollar_bill) or (game_vars.Vars.taken_twenty_dollar_bill):
                yield player.talk("I'm not touching that.")
            else:
                yield player.talk("Holy smokes, twenty dollars!")
                yield player.talk("I better hand it to the saleswoman before someone takes it.")
                yield Delay(4)
                yield player.talk("Hey, a twenty dollar bill! Must be my lucky day.")
                game_vars.Vars.taken_twenty_dollar_bill = True
                yield TwentyDollarBillInv

    @staticmethod
    def look_at_clothes(player, clothes):
        yield player.talk("I don't get today's fashion.")
        yield player.talk("Not enough people wear fedoras these days.")
        yield player.talk("I remember the one Gertrude bought me after we got married.")
        yield player.talk("I can't remember what I got her, but I think it had a tail.")

    @staticmethod
    def look_at_20_dollar_bill(player, bill):
        yield player.talk("I remember when this guy ran for president.")
        yield player.talk("Damn Commie.")

    @staticmethod
    def talk_to_poppy(player, poppy):
        options = [
            "I know it sounds strange, but... Do you know who I am?",
            "How much for a balloon?",
            "What's the spiciest thing you have here?"
            
        ]
        if game_vars.Vars.heard_about_kaboom:
            options.append("How much for Ka-Boom! candies?")

        options = TalkOptions(options)
        yield options
        yield player.talk(options.chosen_option_text)

        if options.chosen_option_index == 0:
            yield poppy.talk("Uncle Poppy knows all the kids.")
            yield poppy.talk("Even the wrinkly ones, who walk funny\nand have an embarrassing stain on their pants.")
            yield player.talk("Uncle Poppy?")
            yield poppy.talk("That's my name! Hey hey!")
            yield player.talk("Please tell me that's not your real name.")
            yield poppy.talk("It didn't use to be, but one visit to the\nCourt Clerk's office and I am 100% Uncle Poppy.")
            yield poppy.talk("That showed the store's owner how\nserious I am about that job.")
            yield poppy.talk("No kid can say I'm not a real uncle now.")
            yield player.talk("I've known you for just about a minute,\nand you are already the creepiest person I've ever met.")
            yield poppy.talk("Hey hey!")

        elif options.chosen_option_index == 1:
            yield poppy.talk("They're free, including the weight.")
            yield player.talk("I thought the weight was meant to keep them in the store.")
            yield poppy.talk("Those balloons are filled with a special super-helium\nI developed at the fun laboratory in my basement.")
            yield player.talk("Fun laborat...")
            yield player.talk("I think I'll just walk slowly towards the exit.")
            yield poppy.talk("Uncle Poppy's basement is full of mystery.")
            yield player.talk("That's what I'm afraid of.")

        elif options.chosen_option_index == 2:
            if poppy.question_index == 0:
                yield player.talk("I could use some bowel movement.")
                yield poppy.talk("We have a large variety of spicy snacks and sauces.")
                yield poppy.talk("Have you tried our Death Valley Tabasco?")
                yield player.talk("Is it very hot?")
                yield poppy.talk("I use it to remove varnish from the counter.")
                yield player.talk("I think I'll pass.")
            elif poppy.question_index == 1:
                yield poppy.talk("There is the Bloody Massacre Chipotle.")
                yield poppy.talk("I use it to get rid of ants.")
                yield player.talk("I'd rather go for something that won't kill me.")
            elif poppy.question_index == 2:
                yield player.talk("Preferably non-lethal.")
                yield poppy.talk("We have Ka-Boom! candies.")
                yield poppy.talk("They're harmless, as long as you don't\nmix them with water based liquids.")
                game_vars.Vars.heard_about_kaboom = True

            poppy.question_index += 1
            poppy.question_index %= 3

        elif options.chosen_option_index == 3:
            yield poppy.talk("5$ a pack.")
            yield player.talk("Five dollars?!")
            yield player.talk("I could have bought a full tank of gas for that money.")
            yield poppy.talk("You haven't been out of the\nhouse for a loooong time, have you?")
            game_vars.Vars.heard_about_kaboom_price = True

    @staticmethod
    def look_at_poppy(player, poppy):
        yield player.talk("His smile scares me.")

    @staticmethod
    def give_twenty_to_poppy(player, poppy, bill):
        if game_vars.Vars.heard_about_kaboom_price and not game_vars.Vars.bought_kaboom:
            yield player.talk("One pack of Ka-Boom! candies please.")
            yield poppy.talk("There you go.")
            yield poppy.talk("Don't forget not to swim for\nat least an hour after eating those.")
            yield Delay(1)
            yield poppy.talk("Actually, you might wanna stay away from fluids altogether.")
            game_vars.Vars.bought_kaboom = True

            bill.kill()
            yield FifteenDollarsInv
            yield KaboomCandiesInv
        else:
            yield player.talk("I don't know what to buy with this money.")

    @staticmethod
    def look_at_sweets(player, sweets):
        yield player.talk("They've got gummy bears, lollipops,\nchewing gum, chocolate...")
        yield player.talk("Energy bar? That's not a candy, that's sawdust.")

    @staticmethod
    def look_at_kaboom(player, kaboom):
        yield player.talk("Ka-Boom! Candies: Pop Rocks plus gingivitis.\nVery bad idea.")

    @staticmethod
    def look_at_spicy(player, spicy):
        yield player.talk("One of these bottles have a picture\nof a burning skull melting in lava.")
        yield player.talk("The label says:\n\"Do Not Remove Cap Without Safety Goggles\"")

    @staticmethod
    def look_at_candy_store_balloons(player, balloons):
        yield player.talk("They're tied to a weight on the counter.\nIt keeps them from flying to the ceiling.")

    @staticmethod
    def make_girl_return(player, store, regular_action):
        for item in regular_action:
            yield item

        if not game_vars.Vars.girl_in_store:
            store.girl = LeShirtGirl(True)
            store.add_npc(store.girl, 40)
            yield Delay(0.5)
            store.girl.walk_back()
            yield Delay(3.4)
            # in case we were skipped
            store.girl.walking = False
            store.girl.rect.centerx = store.girl.START_X
            store.girl.set_straight()
            game_vars.Vars.girl_in_store = True

    @staticmethod
    def talk_to_girl(player, girl):
        options = TalkOptions([
            "I know it sounds strange, but... Do you know who I am?",
            "I'm looking for something not too expensive?",
            "How can I look, as you kids call it, hip and groovy?",
            "Have you got any more gum?"
        ])

        yield options
        yield player.talk(options.chosen_option_text)

        if options.chosen_option_index == 0:
            yield girl.talk("Why? Are you like famous or something?")
            yield player.talk("Well...")
            yield girl.talk("OMG! Are you Justin Bieber?")
            yield player.talk("Is that a cartoon character?")
            yield girl.talk("Psh.")

        elif options.chosen_option_index == 1:
            yield girl.talk("How would you define \"not too expensive\"?")
            yield player.talk("About four dollars, tops.")
            yield girl.talk("Riiiight... Am I on TV? Is this, like, candid camera?")
            yield player.talk("No...")
            yield girl.talk("Wait, are you Ashton Kutcher?")
            yield player.talk("That sounds like the name of a sports car.")
            
        elif options.chosen_option_index == 2:
            yield girl.talk("Sorry, I don't speak Yiddish.")
            yield player.talk("I mean do you have any clothes that\nwould make me feel younger.")
            yield player.talk("I figured it might help to boost my memory.")
            yield girl.talk("Do you have problems with your memory?")
            yield player.talk("Is this store air conditioned?")
            yield girl.talk("Not at the moment.")
            yield player.talk("Then I must have forgotten to put on underwear today.")

        elif options.chosen_option_index == 3:
            yield girl.talk("Are you hitting on me?")
            yield girl.talk("Cause I don't normally go for older guys.")
            yield girl.talk("I mean, I date guys older than me,\nbut nothing mesozoic, no offense.")
            yield player.talk("None taken.")
            yield girl.talk("Although, there was this one time I got really\ndrunk and woke up in the deputy mayor's car...")
            yield girl.talk("Actually, I promised my lawyer not to talk about it.")

    @staticmethod
    def look_at_girl(player, girl):
        yield player.talk("All she does is chew gum and answer the phone.")
        yield player.talk("Even I can do that.")
        yield Delay(1)
        yield player.talk("Well, except for the chewing part.")

#    @staticmethod
#    def talk_to_leshirt_girl(player, girl):
#        yield girl.talk("Good day, Sir. How may I help you?")
#        yield player.talk("Could you...")
#        yield TalkText("*RING*", (255, 255, 0), pygame.Rect(194, 120, 1, 1))
#        yield Delay(0.5)
#        yield TalkText("*RING*", (255, 255, 0), pygame.Rect(194, 120, 1, 1))

#    @staticmethod
#    def talk_to_larry(player, larry):
#        yield player.talk("Hi.")
#        yield Delay(0.5)
#        yield larry.talk("What do you want?")
#
#        while True:
#            options = TalkOptions([
#                "I'm selling these fine leather jackets.",
#                "What's your name?",
#                "How do you solve King's Quest 3?",
#                "Bye."
#            ])
#            yield options
#            yield player.talk(options.chosen_option_text)
#
#            if options.chosen_option_index == 0:
#                yield larry.talk("Look behind you. A three-headed monkey.")
#            elif options.chosen_option_index == 1:
#                yield larry.talk("I'm Larry; Larry Laffer.")
#            elif options.chosen_option_index == 2:
#                yield larry.talk("You don't. You die.")
#                yield player.talk("Snap.")
#            elif options.chosen_option_index == 3:
#                yield larry.talk("See ya.")
#                break
#
#    @staticmethod
#    def look_at_larry(player, larry):
#        yield player.talk("It looks like Leisure Suit Larry.")
#
#    @staticmethod
#    def use_larry(player, larry):
#        yield player.talk("That sounds dangerous.")
#
#    @staticmethod
#    def look_at_handgun(player, handgun):
#        yield player.talk("It looks like a perfectly ordinary street handgun.")
#
#    @staticmethod
#    def talk_to_handgun(player, handgun):
#        yield player.talk("Han shot first.")
#        yield Delay(2)
#        player.set_stand_dir(DIR_FRONT)
#        yield player.talk("Sorry, couldn't think of anything.")
#
#    @staticmethod
#    def pick_up_handgun(player, handgun):
#        yield player.talk("Okay, I guess.")
#        handgun.kill()
#        game_vars.Vars.handgun_taken = True
#        yield HandgunInv
#
#    @staticmethod
#    def look_at_bowling_ball(player, ball):
#        yield player.talk("I can't fit my fingers in there.")
#        yield Delay(0.8)
#        player.set_stand_dir(DIR_FRONT)
#        yield Delay(0.8)
#        yield player.talk("That's what she said.")
#
#    @staticmethod
#    def use_bowling_ball(player, ball):
#        yield ObjectInHand(ball)
#
#    @staticmethod
#    def use_candy(player, candy):
#        yield player.talk("*POP* *POP* *POP*")
#        candy.kill()
#
#    @staticmethod
#    def look_at_no_littering_sign(player, object):
#        yield player.talk('"Do Not Litter"')

    @staticmethod
    def clear_object_in_hand(regular_action):
        yield ObjectInHand(None) # Must do this...

        for item in regular_action:
            yield item

    @classmethod
    def action(cls, verb, player, object, object_in_hand):
        ret = cls.action2(verb, player, object, object_in_hand)

        if isinstance(object, Clothes) or isinstance(object, CashRegister):
            ret = cls.make_girl_return(player, object.store, ret)

        if verb == VERB_USE and object_in_hand is not None:
            ret = cls.clear_object_in_hand(ret)

        return ret

    @classmethod
    def action2(cls, verb, player, object, object_in_hand):
#        if isinstance(object, Larry):
#            if verb == VERB_TALK:
#                return cls.talk_to_larry(player, object)
#            elif verb == VERB_LOOK:
#                return cls.look_at_larry(player, object)
#            elif verb == VERB_USE and object_in_hand is None:
#                return cls.use_larry(player, object)
#        if isinstance(object, HandgunPickup):
#            if verb == VERB_TALK:
#                return cls.talk_to_handgun(player, object)
#            elif verb == VERB_LOOK:
#                return cls.look_at_handgun(player, object)
#            elif verb == VERB_USE and object_in_hand is None:
#                return cls.pick_up_handgun(player, object)
#
#        if isinstance(object, BowlingBallInv):
#            if verb == VERB_LOOK:
#                return cls.look_at_bowling_ball(player, object)
#            elif verb == VERB_USE and object_in_hand is None:
#                return cls.use_bowling_ball(player, object)
#
#        if isinstance(object, NoLitteringSign):
#            if verb == VERB_LOOK:
#                return cls.look_at_no_littering_sign(player, object)

        
        if isinstance(object, RuinedIDCardInv):
            if verb == VERB_LOOK:
                return cls.look_at_id_card(player, object)

        elif object.name == "old man":
            if verb == VERB_LOOK:
                return cls.look_at_myself(player, object)
            elif verb == VERB_TALK:
                return cls.talk_to_myself(player, object)
            elif verb == VERB_USE and object_in_hand is None:
                return cls.use_myself(player, object)

        elif isinstance(object, Pothole):
            if verb == VERB_LOOK:
                return cls.look_at_pothole(player, object)

        elif isinstance(object, Sign):
            if verb == VERB_LOOK:
                return cls.look_at_sign(player, object)

        elif isinstance(object, OptometristDoor):
            if verb == VERB_USE and object_in_hand is None:
                return cls.use_optometrist_door(player, object)

        elif isinstance(object, TouristInformationGuy):
            if verb == VERB_TALK:
                return cls.talk_to_information_guy(player, object)
            elif verb == VERB_LOOK:
                return cls.look_at_information_guy(player, object)
            elif verb == VERB_USE and isinstance(object_in_hand, KaboomCandiesInv):
                return cls.give_candy_to_information_guy(player, object, object_in_hand)
            elif verb == VERB_USE and isinstance(object_in_hand, RuinedIDCardInv):
                return cls.give_id_to_information_guy(player, object, object_in_hand)

        elif isinstance(object, Bookshelf):
            if verb == VERB_LOOK:
                return cls.look_at_bookshelf(player, object)

        elif isinstance(object, TouristsInTouristInformationCenter):
            if verb == VERB_LOOK:
                return cls.look_at_tourists_in_tic(player, object)
            elif verb == VERB_TALK:
                return cls.talk_to_tourists_in_tic(player, object)

        elif isinstance(object, BalloonInTouristInformationCenter):
            if verb == VERB_LOOK:
                return cls.look_at_balloon_in_tic(player, object)

        elif isinstance(object, CoatHanger):
            if verb == VERB_LOOK:
                return cls.look_at_coat_hanger(player, object)

        elif isinstance(object, Coat):
            if verb == VERB_LOOK:
                return cls.look_at_coat(player, object)
            elif verb == VERB_USE and object_in_hand is None:
                return cls.pick_up_coat(player, object)

        elif isinstance(object, Clothes):
            if verb == VERB_USE and object_in_hand is None:
                return cls.pick_up_clothes(player, object)
            elif verb == VERB_LOOK:
                return cls.look_at_clothes(player, object)
            
        elif isinstance(object, CashRegister):
            if verb == VERB_LOOK:
                return cls.look_at_cash_register(player, object)
            elif verb == VERB_USE and object_in_hand is None:
                return cls.pick_up_cash_register(player, object)

        elif isinstance(object, LeShirtGirl):
            if verb == VERB_TALK:
                return cls.talk_to_girl(player, object)
            elif verb == VERB_LOOK:
                return cls.look_at_girl(player, object)

        elif isinstance(object, TwentyDollarBillInv):
            if verb == VERB_LOOK:
                return cls.look_at_20_dollar_bill(player, object)

        elif isinstance(object, Poppy):
            if verb == VERB_LOOK:
                return cls.look_at_poppy(player, object)
            elif verb == VERB_USE and isinstance(object_in_hand, TwentyDollarBillInv):
                return cls.give_twenty_to_poppy(player, object, object_in_hand)
            elif verb == VERB_TALK:
                return cls.talk_to_poppy(player, object)

        elif isinstance(object, CandyStoreSweets):
            if verb == VERB_LOOK:
                return cls.look_at_sweets(player, object)

        elif isinstance(object, KaboomCandies):
            if verb == VERB_LOOK:
                return cls.look_at_kaboom(player, object)

        elif isinstance(object, SpicyStuff):
            if verb == VERB_LOOK:
                return cls.look_at_spicy(player, object)

        elif isinstance(object, CandyStoreBalloons):
            if verb == VERB_LOOK:
                return cls.look_at_candy_store_balloons(player, object)

        # default action
        if verb == VERB_TALK:
            return cls.talk_default(player, object)
        elif verb == VERB_LOOK:
            return cls.look_default(player, object)
        elif verb == VERB_USE and object_in_hand is None:
            return cls.use_default(player, object)
        elif verb == VERB_USE and object_in_hand is not None:
            return cls.use_on_default(player, object, object_in_hand)
