from stage import *
import game_vars

################################################################################
#   Stages
################################################################################

class StreetLeft(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "main-street-left.png", "street-map-left.png", "street-foreground-left.png",
                       "mainstreet.ogg", initial_pos, initial_dir)

        self.add_static(TouristInformationCenterSign())
        self.add_static(CandyStoreSign())
        self.add_static(Pothole1())
        self.add_static(Pothole2())

        if not game_vars.Vars.exposition_shown:
            self.busyqueue.insert(0, self.exposition(self.player))
            game_vars.Vars.exposition_shown = True

    @staticmethod
    def exposition(player):
        yield Delay(2)
        yield player.talk("Wait a minute... This isn't Bean World!")
        yield Delay(2)
        yield player.talk("Oh, right. Forgot what year it is.")

        yield Delay(1)
        player.set_stand_dir(DIR_RIGHT)
        yield Delay(1)
        player.set_stand_dir(DIR_FRONT)

        yield Delay(1)
        yield player.talk("I don't remember ever being to this part of town.")
        yield player.talk("How do I get home from here?")
        yield Delay(1)
        yield player.talk("Now that I think of it... WHERE IS HOME?")

class StreetRight(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "main-street-right.png", "street-map-right.png", None,
                       "mainstreet.ogg", initial_pos, initial_dir)

        self.add_ngo(Tree(), 200)
        self.add_static(Pothole3())
        self.add_static(LeShirtSign())
        self.add_static(OptometristSign())
        self.add_static(OptometristDoor())

class Dump(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "dump.png", "dump-map.png", "dump-foreground.png",
                       "Garbage.ogg", initial_pos, initial_dir)

        self.add_static(SmallBox())
        self.add_static(NoLitteringSign())
        self.add_static(Magnet(), 256)

class Park(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "park.png", "park-map.png", None,
                       "Garden.ogg", initial_pos, initial_dir)

        self.add_ngo(ParkFountain(), 82)
        self.add_ngo(ParkSwing(), 82)
        self.add_ngo(ParkSeeSaw(), 137)
        self.add_ngo(ParkSlide(), 207)
        self.add_ngo(ParkHillA(), 194)
        self.add_ngo(ParkHillB(), 134)
        self.add_ngo(ParkHillC(), 134)
        self.add_ngo(ParkHillD(), 82)
        self.add_ngo(ParkHillE(), 40)

class TouristInformationCenter(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "tourist.png", "tourist-map.png", None,
                       "infocenter.ogg", initial_pos, initial_dir)

        self.add_ngo(TICForeground(), 200)
        guy = TouristInformationGuy()
        self.add_npc(guy)
        #self.add_static(BalloonInTouristInformationCenter())
        self.add_static(CoatHanger())
        self.add_static(Coat(guy))
        self.add_static(Bookshelf())
        self.add_npc(TouristsInTouristInformationCenter())

class LeShirt(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "clothing.png", "clothing_bw.png", None,
                       "clothingstore.ogg", initial_pos, initial_dir)

        self.add_ngo(LeShirtFore1(), 70)
        self.add_ngo(LeShirtFore2(), 150)
        self.add_ngo(LeShirtFore3(), 70)

        self.add_static(Clothes(self))
        self.add_static(CashRegister(self))

        self.girl = LeShirtGirl()
        self.add_npc(self.girl, 40)

        self.busyqueue.insert(0, self.enter_store_scene(self.player, self.girl))

        #self.ring_tick = None

    @staticmethod
    def enter_store_scene(player, girl):
        yield Delay(0.75)
        yield girl.talk("Good day, Sir. How may I help you?")
        yield player.talk("Could you...")
        yield TalkText("*RING*", (255, 255, 0), pygame.Rect(275, 120, 1, 1))
        yield Delay(0.5)
        yield TalkText("*RING*", (255, 255, 0), pygame.Rect(275, 120, 1, 1))
        yield girl.talk("Just a minute, sir.")
        girl.walk_away()
        yield Delay(3.4)
        girl.kill()
        game_vars.Vars.girl_in_store = False

class CandyStore(Stage):
    def __init__(self, initial_pos, initial_dir):
        Stage.__init__(self, "candy.png", "candy_bw.png", None,
                       "candystore.ogg", initial_pos, initial_dir)

        self.add_npc(Poppy())
        self.add_static(CandyStoreBalloons())
        self.add_static(CandyStoreSweets())
        self.add_static(KaboomCandies())
        self.add_static(SpicyStuff())
