from NPC import *
from inventoryitem import *
from pickup import *
from static_object import *
from hotspot import *
import math

################################################################################
#   NPCs
################################################################################

class TouristInformationGuy(NPC):
    TALK_COLOR = (255, 0, 0)
    def __init__(self):
        NPC.__init__(self, "tourist information center employee", self.TALK_COLOR,
                     "tourist-guide.png", "tourist-guide-SPEAK.png", (327, 165), (273, 273), DIR_RIGHTBACK)

class TouristsInTouristInformationCenter(NPC):
    TALK_COLOR = (0, 255, 0)
    def __init__(self):
        NPC.__init__(self, "tourist", self.TALK_COLOR,
                     "metal.png", "metal-speak.png", (545, 291), (474, 296), DIR_RIGHT)

class Poppy(NPC):
    TALK_COLOR = (138, 173, 42)
    def __init__(self):
        NPC.__init__(self, "Uncle Poppy", self.TALK_COLOR,
                     "poppy.png", "poppy-SPEAK.png", (541, 207), (485, 308), DIR_RIGHTBACK)
        self.question_index = 0

class LeShirtGirl(NPC):
    TALK_COLOR = (0, 255, 255)
    START_X = 473
    END_X = 240
    FARTHEST_X = 226
    ANIM_SPEED = 10 # bigger = slower
    WALK_ANIM_SPEED = 15 # bigger = slower
    def __init__(self, at_door = False):
        if at_door:
            x = self.END_X
        else:
            x = self.START_X

        self.walk_image = pygame.image.load(os.path.join("data", "saleslady-walk.png"))

        NPC.__init__(self, "LeShirt saleswoman", self.TALK_COLOR,
                     "saleslady.png", "saleslady-speak.png", (x, 170), (417, 232), DIR_RIGHTBACK)

        self.walking = False
        self.walking_flipped = False

        self.orig_image = self.image
        self.orig_walk_image = self.walk_image
        self.orig_stand_image = self.stand_image

    @classmethod
    def walk_trajectory(cls):
        end_x = cls.FARTHEST_X
        n = 500
        walk_traj = [int(round(cls.START_X * (1 - float(t)/n) + end_x * float(t)/n)) for t in range(n)]
        walk_traj += [end_x] * 5
        start_x = cls.FARTHEST_X
        end_x = cls.END_X
        n = 25
        walk_traj += [int(round(start_x * (1 - float(t)/n) + end_x * float(t)/n)) for t in range(n)]
        return walk_traj

    def walk_away(self):
        if not self.walking:
            self.walk_traj = self.walk_trajectory()
            self.tick = pygame.time.get_ticks()
            self.walking = True
            self.walking_flipped = False

    def walk_back(self):
        if not self.walking:
            self.walk_traj = self.walk_trajectory()[::-1]
            self.tick = pygame.time.get_ticks()
            self.walking = True
            self.walking_flipped = False

    def flip(self):
        self.image = pygame.transform.flip(self.image, True, False)
        self.walk_image = pygame.transform.flip(self.walk_image, True, False)
        self.stand_image = pygame.transform.flip(self.stand_image, True, False)

    def set_straight(self):
        self.image = self.orig_image
        self.walk_image = self.orig_walk_image
        self.stand_image = self.orig_stand_image

    def update(self, *args):
        NPC.update(self, *args)

        tick = pygame.time.get_ticks()

        if self.walking:
            if tick - self.tick >= self.ANIM_SPEED:
                delta = (tick - self.tick) / self.ANIM_SPEED
                self.tick += delta * self.ANIM_SPEED

                if delta >= len(self.walk_traj) - 1:
                    x = self.walk_traj[-1]
                    self.walking = False
                else:
                    x = self.walk_traj[delta]
                    self.walk_traj = self.walk_traj[delta+1:]

                self.rect.centerx = x

                if int(len(self.walk_traj) / self.WALK_ANIM_SPEED) % 2 == 0:
                    self.image = self.get_stand_image()
                else:
                    self.image = self.walk_image

                if x == self.FARTHEST_X and not self.walking_flipped:
                    self.flip()
                    self.walking_flipped = True

################################################################################
#   Inventory items
################################################################################

class HandgunInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "handgun", "inventory_0002_Handgun.png", index, vis_index)

class BowlingBallInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "bowling ball", "inventory_0007_Ball.png", index, vis_index)

class RemoteControlInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "remote control", "inventory_0006_Remote.png", index, vis_index)

class RuinedIDCardInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "ID card", "inventory_0000_ID-copy.png", index, vis_index)

class NeedleInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "needle", "inventory_0015_Needle.png", index, vis_index)

class TwentyDollarBillInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "twenty dollar bill", "inventory_0013_20.png", index, vis_index)

class FifteenDollarsInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "fifteen dollars", "inventory_0012_15.png", index, vis_index)

class KaboomCandiesInv(InventoryItem):
    def __init__(self, index, vis_index):
        InventoryItem.__init__(self, "Ka-Boom! candies", "inventory_0011_Kaboom.png", index, vis_index)

################################################################################
#   Pickup items
################################################################################

class HandgunPickup(Pickup):
    def __init__(self, topleft_pos, walkpos = None, walkdir = None):
        Pickup.__init__(self, "handgun", "inventory_0002_Handgun.png", topleft_pos, walkpos, walkdir)

################################################################################
#   Static objects
################################################################################

class SmallBox(StaticObject):
    def __init__(self):
        StaticObject.__init__(self, "small box", "dump-smallbox.png", (330, 234), (448, 325), DIR_LEFT)

class Magnet(StaticObject):
    START_X = 80
    END_X = 95
    ANIM_SPEED = 50 # milliseconds per pixel (the bigger this is, the slower the magnet moves)
    ANIM_STEPS = 100

    def __init__(self):
        StaticObject.__init__(self, "magnet", "dump-magnet.png", (self.START_X, 0), (200, 153), DIR_LEFTFRONT)

        self.current = 0
        self.trajectory = [round(self.START_X + (self.END_X - self.START_X) * math.sin((2 * math.pi) / self.ANIM_STEPS * i))
                           for i in range(self.ANIM_STEPS)]
        self.tick = None

    def update(self, *args):
        tick = pygame.time.get_ticks()

        if self.tick is None:
            self.tick = tick
            return
        
        delta = int((tick - self.tick) / self.ANIM_SPEED)
        if delta == 0:
            return

        self.tick = tick
        self.current += delta
        self.current %= self.ANIM_STEPS

        self.rect.left = self.trajectory[self.current]

class BalloonInTouristInformationCenter(StaticObject):
    def __init__(self):
        StaticObject.__init__(self, "balloon", "inventory_0007_Ball.png", (574, 159), (474, 296), DIR_RIGHT)

################################################################################
#   Hot spots
################################################################################

class Sign(HotSpot):
    pass

class NoLitteringSign(Sign):
    def __init__(self):
        Sign.__init__(self, "sign", pygame.Rect(349, 326, 60, 38), (432, 346), DIR_LEFTFRONT)

class TouristInformationCenterSign(Sign):
    def __init__(self):
        Sign.__init__(self, "sign", pygame.Rect(52, 115, 212, 38), (151, 320), DIR_BACK)

class CandyStoreSign(Sign):
    def __init__(self):
        Sign.__init__(self, "sign", pygame.Rect(387, 158, 198, 42), (482, 320), DIR_BACK)

class CandyStoreBalloons(PickupHotSpot):
    def __init__(self):
        PickupHotSpot.__init__(self, "balloons", pygame.Rect(571, 74, 51, 50), (439, 318), DIR_RIGHTBACK)

class CandyStoreSweets(PickupHotSpot):
    def __init__(self):
        PickupHotSpot.__init__(self, "sweets", pygame.Rect(311, 73, 76, 112), (352, 225), DIR_BACK)

class KaboomCandies(PickupHotSpot):
    def __init__(self):
        PickupHotSpot.__init__(self, "Ka-Boom! candies", pygame.Rect(122, 62, 79, 127), (163, 225), DIR_BACK)

class SpicyStuff(PickupHotSpot):
    def __init__(self):
        PickupHotSpot.__init__(self, "spicy stuff", pygame.Rect(28, 38, 83, 172), (137, 236), DIR_LEFT)

class Pothole(HotSpot):
    pass

class Pothole1(Pothole):
    def __init__(self):
        Pothole.__init__(self, "pothole", pygame.Rect(307, 339, 46, 17), (330, 320), DIR_FRONT)

class Pothole2(Pothole):
    def __init__(self):
        Pothole.__init__(self, "pothole", pygame.Rect(417, 341, 60, 16), (447, 320), DIR_FRONT)

class Pothole3(Pothole):
    def __init__(self):
        Pothole.__init__(self, "pothole", pygame.Rect(46, 349, 54, 16), (76, 320), DIR_FRONT)

class LeShirtSign(Sign):
    def __init__(self):
        Sign.__init__(self, "sign", pygame.Rect(99, 151, 75, 28), (137, 320), DIR_BACK)

class OptometristSign(Sign):
    def __init__(self):
        Sign.__init__(self, "sign", pygame.Rect(333, 57, 255, 54), (476, 320), DIR_BACK)

class OptometristDoor(HotSpot):
    def __init__(self):
        HotSpot.__init__(self, "door", pygame.Rect(402, 170, 37, 128), (420, 310), DIR_BACK)

class CoatHanger(HotSpot):
    def __init__(self):
        HotSpot.__init__(self, "coat hanger", pygame.Rect(58, 185, 38, 139), (129, 322), DIR_LEFT)

class Coat(PickupHotSpot):
    def __init__(self, employee):
        PickupHotSpot.__init__(self, "coat", pygame.Rect(90, 195, 22, 76), (129, 322), DIR_LEFT)
        self.employee = employee

class Bookshelf(HotSpot):
    def __init__(self):
        HotSpot.__init__(self, "bookshelf", pygame.Rect(260, 83, 138, 23), (273, 273), DIR_RIGHTBACK)
        self.book_num = 0

class Clothes(PickupHotSpot):
    def __init__(self, store):
        PickupHotSpot.__init__(self, "pile of clothes", pygame.Rect(452, 99, 47, 37), (444, 228), DIR_RIGHTBACK)
        self.store = store

class CashRegister(PickupHotSpot):
    def __init__(self, store):
        PickupHotSpot.__init__(self, "cash register", pygame.Rect(107, 174, 52, 35), (103, 253), DIR_RIGHTFRONT)
        self.store = store

################################################################################
#   Non-game objects (things the player can't interact with)
################################################################################

class Tree(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (0, 0), "street-foreground-right.png")

class TICForeground(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (0, 0), "tourist-foreground.png")

class LeShirtFore1(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (0, 0), "clothing-foreground1.png")

class LeShirtFore2(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (0, 0), "clothing-foreground2.png")

class LeShirtFore3(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (0, 0), "clothing-foreground3.png")

class ParkFountain(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (270, 13), "park_0000_fountain270x13.png")

class ParkSwing(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (34, 37), "park_0001_swing34x37.png")

class ParkSeeSaw(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (33, 183), "park_0002_seesaw33x183.png")

class ParkSlide(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (0, 228), "park_0003_slide0x228.png")

class ParkHillA(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (235, 299), "park_0004_hilla235x299.png")

class ParkHillB(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (464, 232), "park_0005_hillb464x232.png")

class ParkHillC(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (595, 202), "park_0006_hillc595x202.png")

class ParkHillD(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (562, 167), "park_0007_hilld562x167.png")

class ParkHillE(NonGameObject):
    def __init__(self):
        NonGameObject.__init__(self, (569, 128), "park_0008_hille569x128.png")

