import os
import pygame
from pygame.locals import *
from game_consts import *

class ShowScreen(object):
    LOOPS = -1
    def __init__(self, screen, image_filename, music_filename):
        self.screen = screen

        self.background = pygame.image.load(os.path.join("data", image_filename))
        self.music = pygame.mixer.Sound(os.path.join("data", music_filename))
        self.display_flags = 0
        self.quit_enabled = True

    def update(self):
        pass
    
    def play(self):
        quit = False

        self.music1 = self.music.play(loops = self.LOOPS)

        self.screen.blit(self.background, (0, 0))
        pygame.display.update()

        while not quit:
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYUP and event.key == K_ESCAPE):
                    quit = True

                elif event.type == KEYUP and event.key == K_f:
                    self.display_flags ^= pygame.FULLSCREEN
                    self.screen = pygame.display.set_mode(SCREEN_SIZE, self.display_flags, 32)
                    self.screen.blit(self.background, (0, 0))
                    pygame.display.update()

                elif event.type == MOUSEBUTTONDOWN and event.button == LEFT_BUTTON:
                    if self.quit_enabled:
                        quit = True

                elif event.type == MOUSEBUTTONDOWN and event.button == RIGHT_BUTTON:
                    if self.quit_enabled:
                        quit = True

            self.update()

        self.music1.stop()

class IntroScreen(ShowScreen):
    def __init__(self, screen):
        ShowScreen.__init__(self, screen, "opening-screen.png", "intro.ogg")

class GameOverScreen(ShowScreen):
    LOOPS = 0
    SECONDS_PER_CREDIT = 10
    def __init__(self, screen):
        ShowScreen.__init__(self, screen, "white-house.jpg", "hailtothecheif.ogg")
        self.started_tick = None
        self.idx = 0

    def change(self, img):
        if self.idx == 0:
            self.music1.stop()
            self.music2 = pygame.mixer.Sound(os.path.join("data", "credits.ogg")).play(loops = -1)
            
        self.idx += 1
        self.background = pygame.image.load(os.path.join("data", img))
        self.screen.blit(self.background, (0, 0))
        pygame.display.update()

    def update(self):
        tick = pygame.time.get_ticks()
        if self.started_tick is None:
            self.started_tick = tick
        else:
            if 21000 <= tick - self.started_tick and self.idx == 0:
                self.change("credits1.png")
            if 21000 + 1000*1*self.SECONDS_PER_CREDIT <= tick - self.started_tick and self.idx == 1:
                self.change("credits2.png")
            if 21000 + 1000*2*self.SECONDS_PER_CREDIT <= tick - self.started_tick and self.idx == 2:
                self.change("credits3.png")
            if 21000 + 1000*3*self.SECONDS_PER_CREDIT <= tick - self.started_tick and self.idx == 3:
                self.change("credits4.png")
            if 21000 + 1000*4*self.SECONDS_PER_CREDIT <= tick - self.started_tick and self.idx == 4:
                self.change("credits5.png")
            if 21000 + 1000*5*self.SECONDS_PER_CREDIT <= tick - self.started_tick and self.idx == 5:
                self.change("credits6.png")
            if 21000 + 1000*6*self.SECONDS_PER_CREDIT <= tick - self.started_tick and self.idx == 6:
                self.change("credits7.png")
                self.quit_enabled = True