from inventoryitem import *
from game_consts import *
import game_vars

class InventoryPane(object):
    POSITION = (0, 394)
    SIZE = (640, 86)
    RECT = pygame.Rect(POSITION[0], POSITION[1], SIZE[0], SIZE[1])

    def __init__(self):
        self.background = pygame.image.load(os.path.join("data", "inventory-blank.png"))
        self.items = pygame.sprite.RenderUpdates()
        self.num_items_taken = 0
        
        for itemclass in game_vars.Vars.items:
            self.add_item(itemclass)

    def add_item(self, itemclass):
        self.num_items_taken += 1
        self.items.add(itemclass(self.num_items_taken, len(self.items)))

    def draw_background(self, screen):
        screen.blit(self.background, self.POSITION)

    def update(self, *args):
        # Sort the items (visual index) by logical index
        indexes = sorted([item.index for item in self.items])
        classes = [None] * len(self.items)

        for item in self.items:
            item.set_vis_index(indexes.index(item.index))
            classes[indexes.index(item.index)] = type(item)

        self.items.update(*args)
        game_vars.Vars.items = classes

    def draw(self, screen):
        self.draw_background(screen)
        return self.items.draw(screen)

    def hitobject(self, pos):
        for obj in self.items:
            if obj.rect.collidepoint(pos):
                return obj

        return None

    def click(self, pos, verb):
        if verb == VERB_WALK:
            return
        
        hitobject = self.hitobject(pos)
        if hitobject is None:
            return

        return hitobject