from game_script import *

class Player(Character):
    MILLISECONDS_PER_PIXEL = 13 # the bigger this is, the slower the player walks
    NONWALKABLE_COLOR = (0, 0, 0)
    TALK_COLOR = (255, 255, 255)
    WALK_ANIM_SPEED = 150 # milliseconds to change image - the bigger this is, the slower the player is animated

    def __init__(self, initial_pos, initial_dir):
        self.stand_images = {
            DIR_FRONT      : pygame.image.load(os.path.join("data", "oldman-front.png")),
            DIR_BACK       : pygame.image.load(os.path.join("data", "oldman-back.png")),
            DIR_LEFT       : pygame.image.load(os.path.join("data", "oldman-left.png")),
            DIR_RIGHT      : pygame.image.load(os.path.join("data", "oldman-right.png")),
            DIR_RIGHTBACK  : pygame.image.load(os.path.join("data", "oldman-45-right-back.png")),
            DIR_RIGHTFRONT : pygame.image.load(os.path.join("data", "oldman-45-right.png")),
            DIR_LEFTBACK   : pygame.image.load(os.path.join("data", "oldman-45-left-back.png")),
            DIR_LEFTFRONT  : pygame.image.load(os.path.join("data", "oldman-45-left.png"))
        }

        self.walkanim_images = {
            DIR_FRONT      : [pygame.image.load(os.path.join("data", "oldman-anim-front-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_BACK       : [pygame.image.load(os.path.join("data", "oldman-anim-back-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_LEFT       : [pygame.image.load(os.path.join("data", "oldman-anim-left-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_RIGHT      : [pygame.image.load(os.path.join("data", "oldman-anim-right-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_RIGHTBACK  : [pygame.image.load(os.path.join("data", "oldman-anim-backright-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_RIGHTFRONT : [pygame.image.load(os.path.join("data", "oldman-anim-frontright-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_LEFTBACK   : [pygame.image.load(os.path.join("data", "oldman-anim-backleft-%d.png" % i)) for i in (1, 2, 3, 4)],
            DIR_LEFTFRONT  : [pygame.image.load(os.path.join("data", "oldman-anim-frontleft-%d.png" % i)) for i in (1, 2, 3, 4)]
        }

        self.talk_images = {
            DIR_FRONT      : pygame.image.load(os.path.join("data", "oldman-front-talk.png")),
            DIR_BACK       : pygame.image.load(os.path.join("data", "oldman-back-talk.png")),
            DIR_LEFT       : pygame.image.load(os.path.join("data", "oldman-left-talk.png")),
            DIR_RIGHT      : pygame.image.load(os.path.join("data", "oldman-right-talk.png")),
            DIR_RIGHTBACK  : pygame.image.load(os.path.join("data", "oldman-45-right-back-talk.png")),
            DIR_RIGHTFRONT : pygame.image.load(os.path.join("data", "oldman-45-right-talk.png")),
            DIR_LEFTBACK   : pygame.image.load(os.path.join("data", "oldman-45-left-back-talk.png")),
            DIR_LEFTFRONT  : pygame.image.load(os.path.join("data", "oldman-45-left-talk.png"))
        }

        self.set_stand_dir(initial_dir)

        Character.__init__(self, "old man", self.TALK_COLOR)

        self.rect = self.image.get_rect()
        self.rect.midbottom = initial_pos
        self.walktrajectory = []
        self.walk_color = (255, 255, 255)
        self.post_walk_standdir = None
        self.action_verb = None
        self.action_object = None

    def get_talk_image(self):
        return self.talk_images[self.stand_dir]

    def get_stand_image(self):
        return self.stand_images[self.stand_dir]

    def set_stand_dir(self, stand_dir):
        self.stand_dir = stand_dir
        game_vars.Vars.player_dir = stand_dir
        self.image = self.stand_images[self.stand_dir]

    def update_walk_anim(self):
        anim_idx = int(pygame.time.get_ticks() / self.WALK_ANIM_SPEED) % len(self.walkanim_images[self.stand_dir])
        self.image = self.walkanim_images[self.stand_dir][anim_idx]

    def walk(self, dest, walkmask, post_walk_standdir):
        if self.rect.midbottom == dest:
            # Already there
            if post_walk_standdir is not None:
                self.set_stand_dir(post_walk_standdir)
            return

        self.post_walk_standdir = post_walk_standdir

        # Plan a walk trajectory
        # First figure out the direction vector

        curx, cury = self.rect.midbottom
        destx, desty = dest

        dirvecx, dirvecy = destx - curx, desty - cury

        # Normalize
        norm = math.sqrt(dirvecx * dirvecx + dirvecy * dirvecy)
        dirvecx = dirvecx / norm
        dirvecy = dirvecy / norm

        # Pick stand direction according to the direction vector...
        # Get the angle
        theta = math.atan2(-dirvecy, dirvecx)
        if -math.pi / 8 <= theta <= math.pi / 8:
            self.set_stand_dir(DIR_RIGHT)
        elif math.pi / 8 <= theta <= math.pi * 3 / 8:
            self.set_stand_dir(DIR_RIGHTBACK)
        elif math.pi * 3 / 8 <= theta <= math.pi * 5 / 8:
            self.set_stand_dir(DIR_BACK)
        elif math.pi * 5 / 8 <= theta <= math.pi * 7 / 8:
            self.set_stand_dir(DIR_LEFTBACK)
        elif -math.pi * 3 / 8 <= theta <= -math.pi / 8:
            self.set_stand_dir(DIR_RIGHTFRONT)
        elif -math.pi * 5 / 8 <= theta <= -math.pi * 3 / 8:
            self.set_stand_dir(DIR_FRONT)
        elif -math.pi * 7 / 8 <= theta <= -math.pi * 5 / 8:
            self.set_stand_dir(DIR_LEFTFRONT)
        else: # less than -7/8 pi or more than 7/8 pi
            self.set_stand_dir(DIR_LEFT)

        # Plan a trajectory
        self.walktrajectory = []

        def add_pixel(pos):
            posx, posy = pos
            if posx >= walkmask.get_width() or posy >= walkmask.get_height():
                return False
            r, g, b = walkmask.get_at(pos)[:3]
            if (r, g, b) == self.NONWALKABLE_COLOR:
                return False

            self.walktrajectory += [(pos, (r, g, b))]
            return True

        for i in range(1, int(norm)):
            pos = (int(curx + i * dirvecx), int(cury + i * dirvecy))
            if add_pixel(pos) == False:
                # Delete the last 10 pixels and break
                self.walktrajectory = self.walktrajectory[:-10]
                break
        else:
            # didn't "break" above
            add_pixel(dest)
            
        self.walktick = pygame.time.get_ticks()

    def set_action(self, verb, object):
        self.action_verb = verb
        self.action_object = object

    def is_walkable_color(self):
        # gray, but not black
        (r, g, b) = self.walk_color
        if (r, g, b) == self.NONWALKABLE_COLOR:
            return False
        return r == g == b

    def update(self, *args):
        Character.update(self, *args)
        
        if self.walktrajectory:
            self.update_walk_anim()
            tick = pygame.time.get_ticks()

            if tick - self.walktick < self.MILLISECONDS_PER_PIXEL:
                return

            while len(self.walktrajectory) > 0 and tick - self.walktick >= self.MILLISECONDS_PER_PIXEL:
                self.rect.midbottom, self.walk_color = self.walktrajectory[0]
                game_vars.Vars.player_pos = self.rect.midbottom
                if self.is_walkable_color():
                    self.walktrajectory = self.walktrajectory[1:]
                else:
                    self.walktrajectory = []
                    self.set_action(None, None)
                self.walktick += self.MILLISECONDS_PER_PIXEL

            if not self.walktrajectory:
                # Just done walking. De-animate.
                if self.post_walk_standdir is not None:
                    self.set_stand_dir(self.post_walk_standdir)
                    self.post_walk_standdir = None
                else:
                    self.stand_still() # De-animate somehow!

        if not self.walktrajectory:
            # Just done walking and about to do action...?
            # If we have an action, do it
            busyqueue, object_in_hand = args
            self.try_do_action(busyqueue, object_in_hand)

    def try_do_action(self, busyqueue, object_in_hand):
        if self.action_verb is None:
            return

        action_verb, action_object = self.action_verb, self.action_object
        self.set_action(None, None)

        if not isinstance(action_object, Player) and self.rect.midbottom != action_object.walkpos:
            return

        action = self.do_action(action_verb, action_object, object_in_hand)
        if action is not None:
            busyqueue.insert(0, action)

    def do_action(self, action_verb, action_object, object_in_hand):
        return GameScript.action(action_verb, self, action_object, object_in_hand)

    def use_item(self, action_verb, inv_item, object_in_hand):
        # Going to use an item? First STOP WALKING, IDIOT.
        self.walktrajectory = []
        self.set_action(None, None)

        return GameScript.action(action_verb, self, inv_item, object_in_hand)
        