from object_in_hand import *
from player import *
from NPC import *

class Stage(object):
    POSITION = (0, 0)
    RECT = pygame.Rect(0, 0, 640, 375)

    def __init__(self, background_filename, mask_filename, foreground_filename, background_music_filename,
                 initial_player_pos, initial_player_dir):
        self.background = pygame.image.load(os.path.join("data", background_filename))
        if foreground_filename is None:
            self.foreground = None
        else:
            self.foreground = pygame.image.load(os.path.join("data", foreground_filename))
        self.walkmask = pygame.image.load(os.path.join("data", mask_filename))

        self.background_music_filename = background_music_filename

        self.player = Player(initial_player_pos, initial_player_dir)
        self.players = pygame.sprite.RenderUpdates(self.player)
        self.npcs_z = {}
        self.pickups_z = {}
        self.statics_z = {}
        self.ngo_z = {} # non-game-objects

        self.busyqueue = []
        self.busysprites = pygame.sprite.RenderUpdates()

        self.busy = False

    def add_generic(self, group_z, object, z):
        if group_z.has_key(z):
            group_z[z].add(object)
        else:
            group_z[z] = pygame.sprite.RenderUpdates(object)

    def add_npc(self, npc, z = 255):
        self.add_generic(self.npcs_z, npc, z)
    
    def add_pickup(self, pickup, z = 255):
        self.add_generic(self.pickups_z, pickup, z)

    def add_static(self, static, z = 255):
        self.add_generic(self.statics_z, static, z)

    def add_ngo(self, ngo, z = 255):
        self.add_generic(self.ngo_z, ngo, z)

    def is_busy(self):
        #return not (len(self.busysprites) == 0 and self.busyqueue == [])
        return self.busy

    def draw_background(self, screen):
        screen.blit(self.background, self.POSITION)

    def draw_foreground(self, screen):
        if not (self.foreground is None):
            screen.blit(self.foreground, self.POSITION)

    def update(self, object_in_hand):
        """
        Returns the TalkOptions sprite, or an RGB color tuple for transition, or an inventory item class,
        or an object in hand, or None
        """

        for collection in [self.npcs_z, self.pickups_z, self.statics_z, self.ngo_z]:
            for group in collection.values():
                group.update(self.busyqueue)

        self.players.update(self.busyqueue, object_in_hand)

        if not self.player.is_walkable_color():
            return self.player.walk_color

        self.busysprites.update()

        while len(self.busysprites) == 0:
            if self.busyqueue:
                try:
                    result = self.busyqueue[0].next()

                    if isinstance(result, BusySprite) or isinstance(result, TalkOptions):
                        self.busy = True
                        self.busysprites.add(result)
                        self.busysprites.update()

                    if isinstance(result, TalkOptions) or isinstance(result, type) or isinstance(result, ObjectInHand):
                        return result

                except StopIteration:
                    self.busyqueue.pop()
            else:
                break

        if len(self.busysprites) == 0:
            self.busy = False

        return None

    def draw(self, screen):
        self.draw_background(screen)

        update_rects = []

        # get z-order...
        if self.player.is_walkable_color():
            player_z, player_z, player_z = self.player.walk_color
        else:
            # else? kind of weird, but whatever...
            player_z = 255

        # Draw by z-order
        z_values = set((self.npcs_z.keys() + self.pickups_z.keys() + self.statics_z.keys() + self.ngo_z.keys() + [player_z]))
        for z in sorted(z_values):
            for collection in [self.npcs_z, self.pickups_z, self.statics_z, self.ngo_z]:
                if collection.has_key(z):
                    update_rects += collection[z].draw(screen)
            if z == player_z:
                update_rects += self.players.draw(screen)

        self.draw_foreground(screen)

        update_rects += self.busysprites.draw(screen) # drawn even over the foreground (i.e. talk texts...)
        
        return update_rects

    def click(self, pos, verb):
        # is the pos even in the stage?
        if not self.RECT.collidepoint(*pos):
            return

        # returns True to change back to walk verb
        change_back_to_walk_verb = False
        hitobject = self.hitobject(pos)
        walkdir = None

        if isinstance(hitobject, Player) and verb == VERB_WALK:
            hitobject = None

        if hitobject != None and hitobject.walkpos != None:
            pos = hitobject.walkpos
            walkdir = hitobject.walkdir # should exist and != None since walkpos != None

        if verb != VERB_WALK and hitobject != None and hitobject.walkpos != None:
            self.player.set_action(verb, hitobject) # deferred to when we walk there
        elif verb != VERB_WALK and isinstance(hitobject, Player):
            self.player.set_action(verb, hitobject) # deferred to very soon, since we won't be walking
        else:
            self.player.set_action(None, None) # do not act on anything
            change_back_to_walk_verb = True # in fact...

        if not isinstance(hitobject, Player):
            # this does the action, either when done walking or immediately if we're there:
            self.player.walk(pos, self.walkmask, walkdir)

        return change_back_to_walk_verb

    def hitobject(self, pos):
        for collection in [self.npcs_z, self.pickups_z, self.statics_z]:
            for group in collection.values():
                for obj in group:
                    if obj.rect.collidepoint(pos):
                        return obj

        if self.player.rect.collidepoint(pos):
            return self.player

        return None

    def user_bored(self):
        self.busysprites.update("User bored")

    def use_item(self, verb, inv_item, object_in_hand):
        action = self.player.use_item(verb, inv_item, object_in_hand)
        if action is not None:
            self.busyqueue.insert(0, action)
