import pygame
from pygame.locals import *
import os

class TalkPaneOption(pygame.sprite.Sprite):
    INITIAL_X = 30
    INITIAL_Y = 375
    HEIGHT_PER_OPTION = 26

    OPTION_FONT_SIZE = 22

    OPTION_PREFIX = "@ "

    COLOR_NOT_CHOSEN = (128, 128, 128)
    COLOR_CHOSEN = (255, 255, 255)

    def __init__(self, index, text, talkpane):
        pygame.sprite.Sprite.__init__(self)
        
        self.pos = (self.INITIAL_X, self.INITIAL_Y + self.HEIGHT_PER_OPTION * index)
        self.index = index
        self.text = text
        self.talkpane = talkpane

        self.font = pygame.font.Font(os.path.join("data", "font.ttf"), self.OPTION_FONT_SIZE)
        self.font.set_bold(True)
        self.color = self.COLOR_NOT_CHOSEN

        self.image = None

    def render(self):
        self.image = self.font.render(self.OPTION_PREFIX + self.text, True, self.color, (0, 0, 0))
        self.rect = self.image.get_rect().move(self.pos)

    def update(self, *args):
        if self.image is None:
            self.render()

        if len(args) == 2:
            mouseevent, mousepos = args
            if self.rect.collidepoint(mousepos):
                if mouseevent == MOUSEMOTION:
                    if self.color == self.COLOR_NOT_CHOSEN:
                        self.color = self.COLOR_CHOSEN
                        self.render()
                elif mouseevent == MOUSEBUTTONDOWN:
                    self.talkpane.choose(self)
            else:
                if self.color == self.COLOR_CHOSEN:
                    self.color = self.COLOR_NOT_CHOSEN
                    self.render()