import os
from busysprite import *
from game_consts import *

class TalkText(TimeoutSprite):
    TALK_FONT_SIZE = 20
    INITIAL_TALK_TIME = 0.9 # seconds
    EXTRA_SECONDS_PER_WORD = 0.4
    OUTLINE_COLOR = (0, 0, 0)

    def render_one_line(self, text):
        # render outline
        outline_text = self.font.render(text, False, self.OUTLINE_COLOR)
        # Create surface for text
        surface = pygame.Surface((outline_text.get_width() + 5, outline_text.get_height() + 5), flags=SRCALPHA, depth=32)
        surface.fill((0, 0, 0, 0))

        # Blit it several times to get a 4 pixel wide border
        for x in range(5):
            for y in range(5):
                surface.blit(outline_text, (x, y))

        # Blit it once more with the correct color
        surface.blit(self.font.render(text, False, self.color), (2, 2))
        return surface

    def __init__(self, text, color, character_rect, on_start = None, on_kill = None):
        TimeoutSprite.__init__(self, self.INITIAL_TALK_TIME + (text.count(" ") + 1) * self.EXTRA_SECONDS_PER_WORD, on_start, on_kill)

        self.font = pygame.font.Font(os.path.join("data", "font.ttf"), self.TALK_FONT_SIZE)
        self.font.set_bold(True)
        self.color = color
        self.text = text
        self.char_rect = character_rect

        # Render the lines
        lines = [ self.render_one_line(line) for line in text.split("\n") ]

        # Create a surface for all the lines together
        width = max([line.get_width() for line in lines])
        height = sum([line.get_height() for line in lines])
        self.image = pygame.Surface((width, height), flags=SRCALPHA, depth=32)
        self.image.fill((0, 0, 0, 0))

        # Blit each line
        y = 0
        for line in lines:
            x = (width - line.get_width()) / 2
            self.image.blit(line, (x, y))
            y += line.get_height()

        self.rect = self.image.get_rect()
        self.rect.midbottom = self.char_rect.midtop
        self.rect.move_ip(0, -25)

        # Fix rect being out of the screen
        if self.rect.left < 10:
            self.rect.left = 10
        if self.rect.right > SCREEN_WIDTH - 10:
            self.rect.right = SCREEN_WIDTH - 10
        if self.rect.top < 10:
            self.rect.top = 10
        if self.rect.bottom > SCREEN_HEIGHT - 10: # wtf? this can't happen :)
            self.rect.bottom = SCREEN_HEIGHT - 10
