from inventoryitem import *
from pickup import *
from NPC import *
from player import *

class SmallText(object):
    def __init__(self, text, color):
        self.text = text
        self.color = color

class Textline(pygame.sprite.Sprite):
    FONT_SIZE = 20
    SIZE = (640, 19)
    START_X = 13
    BACK_COLOR = (0, 73, 126)

    RECT = pygame.Rect(0, 375, 640, 19)

    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        self.font = pygame.font.Font(os.path.join("data", "font.ttf"), self.FONT_SIZE)
        self.font.set_bold(True)
        self.text = []

        self.image = pygame.Surface(self.SIZE, depth=32)

        self.rect = self.RECT

    def update(self, *args):
        verb, hitobject, object_in_hand = args
        self.set_text_by_verb(verb, hitobject, object_in_hand)

        self.image.fill(self.BACK_COLOR)
        x = self.START_X

        for smalltext in self.text:
            rendered = self.font.render(smalltext.text, True, smalltext.color, self.BACK_COLOR)
            self.image.blit(rendered, (x, 0))
            x += rendered.get_width()

    def set_text_by_verb(self, verb, dest_object, object_in_hand):
        # Anyway, an inventory item (and the player) are not a walk destination :)
        if verb == VERB_WALK and (isinstance(dest_object, InventoryItem) or isinstance(dest_object, Player)):
            dest_object = None

        # Also, can't use an item on itself
        if verb == VERB_USE and (dest_object is object_in_hand):
            dest_object = None

        # No dest object:
        if dest_object is None:
            if verb == VERB_USE and object_in_hand is not None:
                self.text = [ SmallText("Use ", COLOR_GRAY),
                              SmallText(object_in_hand.name, COLOR_WHITE),
                              SmallText(" on", COLOR_GRAY)]
            else:
                text = { VERB_WALK : "Walk",
                         VERB_TALK : "Talk to",
                         VERB_USE: "Use / Pick up",
                         VERB_LOOK : "Look at"}[verb]
                self.text = [ SmallText(text, COLOR_GRAY) ]

        # Dest object:
        else:
            # Using something on something / Giving something to someone
            if verb == VERB_USE and object_in_hand is not None:
                if isinstance(dest_object, NPC):
                    self.text = [ SmallText("Give ", COLOR_GRAY),
                                  SmallText(object_in_hand.name, COLOR_WHITE),
                                  SmallText(" to ", COLOR_GRAY),
                                  SmallText(dest_object.name, COLOR_WHITE) ]
                else:
                    self.text = [ SmallText("Use ", COLOR_GRAY),
                                  SmallText(object_in_hand.name, COLOR_WHITE),
                                  SmallText(" on ", COLOR_GRAY),
                                  SmallText(dest_object.name, COLOR_WHITE) ]

            # Picking something up
            elif verb == VERB_USE and object_in_hand is None and (isinstance(dest_object, Pickup) or isinstance(dest_object, PickupHotSpot)):
                self.text = [ SmallText("Pick up ", COLOR_GRAY),
                              SmallText(dest_object.name, COLOR_WHITE) ]

            else:
                text = { VERB_WALK : "Walk to ",
                         VERB_TALK : "Talk to ",
                         VERB_USE: "Use ",
                         VERB_LOOK : "Look at "}[verb]
                self.text = [ SmallText(text, COLOR_GRAY),
                              SmallText(dest_object.name, COLOR_WHITE) ]